/* Implementation of polynom operations
 * Author:  Yotam Medini  yotam.medini@gmail.com -- Created: 2010/December/20 
 */

#include "polynom.h"
#include <stdlib.h>
#include <math.h>


const double polynom_epsilon = 1e-6;


polynom_t  *polynom_create(int degree)
{
    int  k;
    polynom_t  *p = malloc(sizeof(polynom_t));
    p->degree = degree;
    p->coefs = malloc((degree + 1)*sizeof(double));
    for (k = 0; k < degree; k++) { p->coefs[k] = 0; }
    p->coefs[degree] = 1;
    return p;
}


polynom_t  *polynom_create_constant(double c0)
{
    polynom_t  *p = polynom_create(0);
    p->coefs[0] = c0;
    return p;
}


void polynom_copy(polynom_t *target, polynom_t *source)
{
    int  i, d = source->degree;
    target->degree = d;
    target->coefs = realloc(target->coefs, (d + 1) * sizeof(double));
    for (i = 0;  i <= d;  i++)
    {
        target->coefs[i] = source->coefs[i];
    }
}


void polynom_destroy(polynom_t *poly)
{
    free(poly->coefs);
    free(poly);
}


double polynom_coef(polynom_t *p, int i)
{
    return (i <= p->degree ? p->coefs[i] : 0.);
}


int polynom_equal(polynom_t *p0, polynom_t *p1)
{
    int       zero = 0;
    /* HomeWork */
    return zero;
}


double  polynom_evaluate(polynom_t *p, double x)
{
    double y = 0;
    /* HomeWork */
    return y;
}


void polynom_print(FILE *f, polynom_t *p)
{
    int  k = p->degree;

    fprintf(f, "%gx^%d", p->coefs[k], k);
    for (k--;  k >= 0;  k--)
    {
        double  a = p->coefs[k];
        char    *sym = "+";
        if (a < 0)
        {
            a = -a;
            sym = "-";
        }
        fprintf(f, " %s %gx^%d", sym, a, k);
    }
}


void polynom_reduce(polynom_t *p)
{
    int  k;

    /* Zero tiny coefficients */
    for (k = p->degree;  k >= 0;  k--)
    {
        if (fabs(p->coefs[k]) < polynom_epsilon)
        {
             p->coefs[k] = 0.;
        }
    }

    /* Reduce degree */ 
    for (k = p->degree;  k > 0 && fabs(p->coefs[k]) < polynom_epsilon;  k--);
    p->degree = k;
}


void polynom_add(polynom_t *result, polynom_t *p, polynom_t *q)
{
    int  i, dmax = (p->degree < q->degree ? q->degree : p->degree);
    result->coefs = realloc(result->coefs, (dmax + 1) * sizeof(double));

    for (i = 0;  i <= dmax;  i++)
    {
        result->coefs[i] = polynom_coef(p, i) + polynom_coef(q, i);
    }
    result->degree = dmax;
    polynom_reduce(result);
}


void polynom_subtract(polynom_t *result, polynom_t *p, polynom_t *q)
{
    polynom_t  *minus1 = polynom_create_constant(-1);
    polynom_t  *minus_q = polynom_create(0);
    polynom_multiply(minus_q, minus1, q);

    polynom_add(result, p, minus_q);

    polynom_destroy(minus1);
    polynom_destroy(minus_q);
}


void polynom_multiply(polynom_t *result, polynom_t *p, polynom_t *q)
{
    int      k, d = p->degree + q->degree;
    double  *new_coefs = malloc((d + 1)*sizeof(double));

    for (k = 0;  k <= d;  k++)
    {
        double  ck = 0;
        int     j;
        for (j = 0;  j <= k;  j++)
        {
            ck += polynom_coef(p, j) * polynom_coef(q, k - j);
        }
        new_coefs[k] = ck;
    }
    result->degree = d;
    free(result->coefs);
    result->coefs = new_coefs;
}


void polynom_divide(polynom_t *q, polynom_t *r, polynom_t *n, polynom_t *d)
{
    /* HomeWork */
}
